/*  This file is part of the KDE 
    Copyright (C) 1998 Thomas Tanghus (tanghus@earthling.net)

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/  

#include "kcolorgrid.moc"

void KColorArray::remove(int idx)
{
  int oldsize = size();
  if(idx >= (int)size())
  {
    warning("KColorArray::remove: Index %d out of range", idx);
    return;
  }
  KColorArray tmp(*this);
  tmp.detach();
  resize(size()-1);
  for(int i = idx; i < oldsize-1; i++)
    at(i) = tmp[i+1];
  //debug("KColorArray::remove() %u\t-\tsize: %d", at(idx), size());
}

void KColorArray::append(uint c)
{
  resize(size()+1);
  at(size()-1) = c;
  //debug("KColorArray::append() %u\t-\tsize: %d", c, size());
}

void KColorGrid::paintCell( QPainter *painter, int row, int col )
{
  //debug("KColorGrid::paintCell()");
  QBrush brush;
  int d = 0;
  int e = 0;
  if(hasgrid)
  {
    d = s;
    if(gridState() == Plain)
      qDrawPlainRect( painter, d, d, cellWidth()-d, cellHeight()-d, QColor("#000000"), 1);
    else
    {
      e = 1;
      qDrawShadePanel( painter, d, d, cellWidth()-d, cellHeight()-d, colorGroup(), true, 1, &brush);
    }
  }
  painter->setPen( gridcolors[ row * numCols() + col] );
  painter->setBrush( QBrush( gridcolors[ row * numCols() + col ] ) );
  painter->drawRect( d+e, d+e, cellWidth()-(2*d)+e, cellHeight()-(2*d)+e );

  //debug("KColorGrid::paintCell() - done");
}

KColorGrid::KColorGrid(QWidget *parent, const char *name, int space)
 : QTableView(parent, name)
{
  initMetaObject();
  s = space;
  setGridState(Plain);
  setGrid(true);
  setTableFlags(Tbl_autoScrollBars);
  setCellWidth(10);
  setCellHeight(10);
  numcolors.resize(0);
  gridcolors.resize(0);
}

void KColorGrid::setNumRows(int n)
{
  //debug("setNumRows");
  gridcolors.resize(n*numCols());
  QTableView::setNumRows(n);
  //debug("setNumRows() - gridcolors: %d  size: %d", gridcolors.size(), numCols()*numRows());
}

void KColorGrid::setNumCols(int n)
{
  //debug("setNumCols");
  if(n < 0)
    return;
  int on = numCols();
  KColorArray gc(gridcolors);
  gc.detach();
  //debug("gc size: %d numrows: %d", gc.size(), numRows());
  gridcolors.resize(n*numRows());
  QTableView::setNumCols(n);
  //debug("numRows: %d", numRows());
  //debug("gridcolor: %d  grid: %d", gridcolors.size(), numRows()*numCols());
  if(numRows() == 0)
    return;

  for(int i = 0; i < numRows(); i++)
  {
    for(int j = 0; j < on; j++)
    {
      //debug("row %d , col %d", i, j);
      if(on <= n)
      {
        //debug("%d %d", (i*numCols())+j, (i*on)+j);
        gridcolors.at((i*numCols())+j) = gc.at((i*on)+j);
      }
      else
        break;
    }
  }


  //debug("setNumCols() - gridcolors: %d  size: %d", gridcolors.size(), numCols()*numRows());
}

void KColorGrid::fill( uint color)
{
  gridcolors.fill(color);
  numcolors.resize(1);
  numcolors.at(0) = color;
  emit colorschanged(numcolors.size(), numcolors.data());
}

void KColorGrid::setColor( int colNum, uint col )
{
  //debug("KColorGrid::setColor");
  uint oldcolor = gridcolors[colNum];
  gridcolors[colNum] = col;

  if(!numcolors.contains(col))
  {
    //debug("KColorGrid::setColor() - adding %u - %d %d %d",
    //  col, qRed(col), qGreen(col), qBlue(col));
    numcolors.append(col);
    //debug("KColorGrid::setColor() - adding done %d", numcolors.size()-1); 
    //numcolors++;
    emit addingcolor(col);
  }

  if(!gridcolors.contains(oldcolor))
  {
    int idx = numcolors.find(oldcolor);
    if(idx != -1)
    {
      //debug("KColorGrid::setColor() - removing %u - %d %d %d",
      //  oldcolor, qRed(oldcolor), qGreen(oldcolor), qBlue(oldcolor));
      numcolors.remove(idx);
      //debug("KColorGrid::setColor() - removing done"); 
      emit colorschanged(numcolors.size(), numcolors.data());
    }
    //numcolors--;
  }

  updateCell( colNum/numCols(), colNum%numCols() );
  //modified = true;    
}


