/*  This file is part of the KDE 
    Copyright (C) 1998 Thomas Tanghus (tanghus@earthling.net)

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/  

#include "kiconedit.moc"

const char aboutstring[] = "KDE Icon Editor\nWritten by Thomas Tanghus <tanghus@earthling.net>\n";

QList<KIconEdit> KIconEdit::windowlist;

KIconEdit::KIconEdit(const char *name, const char *xpm)
 : KTopLevelWidget(name)
{
  initMetaObject();
  windowlist.append(this);
  setCaption(kapp->getCaption());

  enabletoolbar = enablestatusbar = false;
  menubar = 0L;
  toolbar = 0L;
  drawtoolbar = 0L;
  statusbar = 0L;
  mainview = 0L;

  setupMenuBar();
  setupToolBar();
  setupDrawToolBar();
  setupStatusBar();

  icontemplates = new KIconTemplateContainer;
  newicon = new KNewIcon(this);
  connect( newicon, SIGNAL(newicon(const QSize)), SLOT(openBlank(const QSize)));
  connect( newicon, SIGNAL(newicon(const QString)), SLOT(openTemplate(const QString)));
  newicon->hide();

  filepath = xpm;

  mainview = new QWidget(this); //KIconEditView(this);
  QBoxLayout *l = new QHBoxLayout(mainview);
  toolsw = new KIconToolsView(mainview);
  grid = new KIconGrid(mainview, 0, xpm);
  toolsw->setPreview(grid->pixmap());
  setMinimumHeight(toolsw->sizeHint().height());
  l->addWidget(grid);
  l->addWidget(toolsw);
  l->activate();

  connect( toolsw, SIGNAL( newcolor(uint)),
     grid, SLOT(setColorSelection(uint)));
  connect( grid, SIGNAL( changed(const QPixmap &)),
    toolsw, SLOT(setPreview(const QPixmap &)));
  connect( grid, SIGNAL( addingcolor(uint) ),
    toolsw, SLOT(addColor(uint)));
  connect( grid, SIGNAL( colorschanged(uint, uint*) ),
    toolsw, SLOT(addColors(uint, uint*)));
  connect( grid, SIGNAL(opennewwin(const char *)),
           SLOT(openNewWin(const char *)));
  connect(grid, SIGNAL(sizechanged(int, int)),
           SLOT( updateStatusSize(int, int)));
  connect(grid, SIGNAL(scalingchanged(int)),
           SLOT( updateStatusScaling(int)));
  connect(grid, SIGNAL(namechanged(const char *)),
           SLOT( updateStatusName(const char *)));
  connect( grid, SIGNAL( addingcolor(uint) ),
           SLOT(updateStatusColors(uint)));
  connect(grid, SIGNAL(colorschanged(uint, uint*)),
           SLOT( updateStatusColors(uint, uint*)));

  slotActions(ID_DRAW_FREEHAND);
  updateStatusSize(grid->cols(), grid->rows());
  updateStatusScaling(grid->scaling());
  updateStatusName(grid->title());
  uint *c = 0, n = 0;
  n = grid->getColors(c);
  updateStatusColors(n, c);
  setView(mainview);
  show();
}

void KIconEdit::closeEvent(QCloseEvent *e)
//bool KIconEdit::queryExit()
{
  bool cancel = false;
  if (grid->isModified()) 
  {
    int r = KMsgBox::yesNoCancel(this, i18n("Warning"), 
		i18n("The current file has been modified.\nDo you want to save it?"));
    switch(r)
    {
      case 1:
        grid->save(filepath.data());
        break;
      case 2:
        break;
      case 3:
        cancel = true;
        break;
      default:
        break;
    }
  }
  if(!cancel)
  {
    delete toolbar;
    toolbar = 0L;
    delete drawtoolbar;
    drawtoolbar = 0L;
    delete menubar;
    menubar = 0L;
    delete icontemplates;
    icontemplates = 0L;
    windowlist.remove(this);
    if(windowlist.isEmpty())
      kapp->quit();
    e->accept();
  }
  else
    e->ignore();
}

KIconEdit::~KIconEdit()
{
  delete newicon;
}

void KIconEdit::unsaved(bool flag)
{
    setUnsavedData(flag);
}

void KIconEdit::saveProperties(KConfig *config )
{
    config->writeEntry("Name", grid->title());
}


void KIconEdit::readProperties(KConfig *config)
{
    QString entry = config->readEntry("Name", ""); // no default
    if (entry.isEmpty())
	return;
    grid->open( entry.data());
}

QSize KIconEdit::sizeHint()
{
  if(mainview)
    return mainview->sizeHint();
  else
    return QSize(-1, -1);
}

KMenuBar *KIconEdit::setupMenuBar()
{
  menubar = new KMenuBar(this);
  setMenu(menubar);
  CHECK_PTR(menubar);
  file = new QPopupMenu;
  CHECK_PTR(file);
  file->insertItem(i18n("&New window"), ID_FILE_NEWWIN);
  //file->setItemEnabled(ID_FILE_NEWWIN, false);
  file->insertSeparator();
  file->insertItem(i18n("&New"), ID_FILE_NEWFILE);
  file->setAccel(CTRL + Key_N, ID_FILE_NEWFILE);
  file->insertItem(i18n("&Open"), ID_FILE_OPEN);
  file->setAccel(CTRL + Key_O, ID_FILE_OPEN);
  file->insertItem(i18n("&Save"), ID_FILE_SAVE);
  file->setAccel(CTRL + Key_S, ID_FILE_SAVE);
  file->insertItem(i18n("Save &as"), ID_FILE_SAVEAS);
  file->insertSeparator();
  file->insertItem(i18n("&Print"), ID_FILE_PRINT);
  file->setAccel(CTRL + Key_P, ID_FILE_PRINT);
  //file->setItemEnabled(ID_FILE_PRINT, false);
  file->insertSeparator();
  file->insertItem(i18n("&Close"), ID_FILE_CLOSE);
  file->setAccel(CTRL + Key_W, ID_FILE_CLOSE);
  file->insertItem(i18n("&Quit"), ID_FILE_QUIT);
  file->setAccel(CTRL + Key_Q, ID_FILE_QUIT);
  menubar->insertItem(i18n("&File"), file);

  edit = new QPopupMenu;
  CHECK_PTR(edit);
  edit->insertItem(i18n("Cu&t"), ID_EDIT_CUT);
  //edit->setItemEnabled(ID_EDIT_CUT, false);
  edit->insertItem(i18n("&Copy"), ID_EDIT_COPY);
  //edit->setItemEnabled(ID_EDIT_COPY, false);
  edit->insertItem(i18n("&Paste"), ID_EDIT_PASTE);
  //edit->setItemEnabled(ID_EDIT_PASTE, false);
  edit->insertSeparator();
  edit->insertItem(i18n("Clea&r"), ID_EDIT_CLEAR);
  menubar->insertItem(i18n("&Edit"), edit);

  view = new QPopupMenu;
  CHECK_PTR(view);
  view->insertItem(i18n("Zoom &in"), ID_VIEW_ZOOM_IN);
  view->insertItem(i18n("Zoom &out"), ID_VIEW_ZOOM_OUT);
  view->insertSeparator();
  view->insertItem(i18n("Toggle &Grid"), ID_VIEW_TOGGLE_GRID);
  menubar->insertItem(i18n("&View"), view);

  tools = new QPopupMenu;
  CHECK_PTR(tools);
  tools->insertItem(i18n("Freehand"), ID_DRAW_FREEHAND);
  tools->insertItem(i18n("Rectangle"), ID_DRAW_RECT);
  tools->insertItem(i18n("Filled rectangle"), ID_DRAW_RECT_FILL);
  tools->insertItem(i18n("Circle"), ID_DRAW_CIRCLE);
  tools->insertItem(i18n("Ellipse"), ID_DRAW_ELLIPSE);
  tools->insertItem(i18n("Spray"), ID_DRAW_SPRAY);
  tools->insertItem(i18n("Flood fill"), ID_DRAW_FILL);
  tools->insertItem(i18n("Line"), ID_DRAW_LINE);
  tools->insertItem(i18n("Straight line"), ID_DRAW_STRAIGHT);
  tools->insertItem(i18n("Eraser (Transparent)"), ID_DRAW_ERASE);
  //tools->insertSeparator();
  menubar->insertItem(i18n("&Tools"), tools);

  menubar->insertSeparator();
  help = kapp->getHelpMenu(true,i18n(aboutstring));
  menubar->insertItem(i18n("&Help"), help);

  menubar->show();
  connect( menubar, SIGNAL(activated(int)), SLOT(slotActions(int)));
  return menubar;
}

KToolBar *KIconEdit::setupToolBar()
{
  toolbar = new KToolBar(this);
  addToolBar(toolbar);

  toolbar->insertButton(Icon("filenew.xpm"), ID_FILE_NEWFILE, TRUE,
			  klocale->translate("New File"));
  toolbar->insertButton(Icon("fileopen.xpm"),ID_FILE_OPEN, TRUE, 
			  klocale->translate("Open a file"));
  toolbar->insertButton(Icon("filefloppy.xpm"), ID_FILE_SAVE, TRUE, 
			  klocale->translate("Save the file"));
  toolbar->insertSeparator();
  toolbar->insertButton(Icon("fileprint.xpm"),ID_FILE_PRINT, TRUE, 
			  klocale->translate("Print icon"));
  toolbar->insertSeparator();
  toolbar->insertButton(Icon("editcut.xpm"),ID_EDIT_CUT, TRUE, 
			  klocale->translate("Cut"));
  toolbar->insertButton(Icon("editcopy.xpm"),ID_EDIT_COPY, TRUE, 
			  klocale->translate("Copy"));
  toolbar->insertButton(Icon("editpaste.xpm"),ID_EDIT_PASTE, TRUE, 
			  klocale->translate("Paste"));
  toolbar->insertSeparator();
  toolbar->insertButton(Icon("viewmag-.xpm"),ID_VIEW_ZOOM_OUT, TRUE, 
			  klocale->translate("Zoom out"));
  toolbar->insertButton(Icon("viewmag+.xpm"),ID_VIEW_ZOOM_IN, TRUE, 
			  klocale->translate("Zoom in"));
  toolbar->insertSeparator();
  toolbar->insertButton(Icon("grid.xpm"),ID_VIEW_TOGGLE_GRID, TRUE, 
			  klocale->translate("Toggle grid"));
  toolbar->setToggle(ID_VIEW_TOGGLE_GRID, true);
  toolbar->setButton(ID_VIEW_TOGGLE_GRID, true);
    
  toolbar->setBarPos(KToolBar::Top);
  toolbar->show();
  connect( toolbar, SIGNAL(clicked(int)), SLOT(slotActions(int)));

  return toolbar;
}

KToolBar *KIconEdit::setupDrawToolBar()
{
  drawtoolbar = new KToolBar(this);
  addToolBar(drawtoolbar);

  drawtoolbar->insertButton(Icon("pointer.xpm"), ID_DRAW_FIND, TRUE,
			  klocale->translate("Find pixel"));
  drawtoolbar->setToggle(ID_DRAW_FIND, true);
  drawtoolbar->insertButton(Icon("areaselect.xpm"), ID_DRAW_SELECT, TRUE,
			  klocale->translate("Select area"));
  drawtoolbar->setToggle(ID_DRAW_SELECT, true);
  drawtoolbar->insertButton(Icon("line.xpm"), ID_DRAW_LINE, TRUE,
			  klocale->translate("Draw line"));
  drawtoolbar->setToggle(ID_DRAW_LINE, true);
  drawtoolbar->insertButton(Icon("straightline.xpm"), ID_DRAW_STRAIGHT, TRUE,
			  klocale->translate("Draw straight line"));
  drawtoolbar->setToggle(ID_DRAW_LINE, true);
  drawtoolbar->insertButton(Icon("pen.xpm"), ID_DRAW_FREEHAND, TRUE,
			  klocale->translate("Draw freehand"));
  drawtoolbar->setToggle(ID_DRAW_FREEHAND, true);
  drawtoolbar->insertButton(Icon("circle.xpm"),ID_DRAW_CIRCLE, TRUE, 
			  klocale->translate("Draw circle"));
  drawtoolbar->setToggle(ID_DRAW_CIRCLE, true);
  drawtoolbar->insertButton(Icon("rectangle.xpm"),ID_DRAW_RECT, TRUE, 
			  klocale->translate("Draw rectangle"));
  drawtoolbar->setToggle(ID_DRAW_RECT, true);
  drawtoolbar->insertButton(Icon("filledrectangle.xpm"),ID_DRAW_RECT_FILL, TRUE, 
			  klocale->translate("Draw filled rectangle"));
  drawtoolbar->setToggle(ID_DRAW_RECT_FILL, true);
  drawtoolbar->insertButton(Icon("ellipse.xpm"),ID_DRAW_ELLIPSE, TRUE, 
			  klocale->translate("Draw ellipse"));
  drawtoolbar->setToggle(ID_DRAW_ELLIPSE, true);
  drawtoolbar->insertButton(Icon("spraycan.xpm"),ID_DRAW_SPRAY, TRUE, 
			  klocale->translate("Spray"));
  drawtoolbar->setToggle(ID_DRAW_SPRAY, true);
  drawtoolbar->insertButton(Icon("flood.xpm"),ID_DRAW_FILL, TRUE, 
			  klocale->translate("Flood fill"));
  drawtoolbar->setToggle(ID_DRAW_FILL, true);
  drawtoolbar->setToggle(ID_DRAW_SPRAY, true);
  drawtoolbar->insertButton(Icon("eraser.xpm"),ID_DRAW_ERASE, TRUE, 
			  klocale->translate("Erase"));
  drawtoolbar->setToggle(ID_DRAW_ERASE, true);
    
  drawtoolbar->setBarPos(KToolBar::Left);
  drawtoolbar->show();
  connect( drawtoolbar, SIGNAL(clicked(int)), SLOT(slotActions(int)));

  return drawtoolbar;
}

KStatusBar *KIconEdit::setupStatusBar()
{
  statusbar = new KStatusBar(this);
  setStatusBar(statusbar);
  CHECK_PTR(statusbar);
  statusbar->insertItem("Colors:       ", 0);
  statusbar->insertItem(" 32 x 32 ", 1);
  statusbar->insertItem("", 2);

  return statusbar;
}

void KIconEdit::slotActions( int id )
{

  switch(id)
  {
    case ID_FILE_NEWWIN:
    {
      KIconEdit *w = new KIconEdit();
      CHECK_PTR(w);
      debug("New window");
      break;
    }
    case ID_FILE_NEWFILE:
    {
      bool cancel = false;
      if (grid->isModified()) 
      {
        int r = KMsgBox::yesNoCancel(this, i18n("Warning"), 
		i18n("The current file has been modified.\nDo you want to save it?"));
        switch(r)
        {
          case 1:
	    grid->save(filepath.data());
            break;
          case 2:
            break;
          case 3:
            cancel = true;
            break;
          default:
            break;
        }
      }
      if(!cancel)
      {
        newicon->gotoPage(0);
        newicon->show();
        break;
      }
      break;
    }
    case ID_FILE_OPEN:
    {
      bool cancel = false;
      if (grid->isModified()) 
      {
        int r = KMsgBox::yesNoCancel(this, i18n("Warning"), 
		i18n("The current file has been modified.\nDo you want to save it?"));
        switch(r)
        {
          case 1:
	    grid->save(filepath.data());
            break;
          case 2:
            break;
          case 3:
            cancel = true;
            break;
          default:
            break;
        }
      }
      if(!cancel)
      {
	grid->open();
	break;
      }
    }
    case ID_FILE_SAVE:
    {
      debug("KIconEdit - Saving %s", filepath.data());
      grid->save(filepath.data());
      break;
    }
    case ID_FILE_SAVEAS:
      grid->save(0);
      break;
    case ID_FILE_PRINT:
    {
      QPrinter prt;
      if ( prt.setup(this) )
      {
        prt.setCreator("KDE Icon Editor");
        QPainter p;
        p.begin( &prt );
        p.drawPixmap( 0,0, grid->pixmap() );
        p.end();
      }
    break;
    }
    case ID_FILE_CLOSE:
    {
      close();
      break;
    }
    case ID_FILE_QUIT:
    {
      for (KIconEdit *w = windowlist.first(); w ; w = windowlist.next())
        w->close();
      kapp->exit();
      break;
    }
    case ID_EDIT_CUT:
      grid->editCut();
      break;
    case ID_EDIT_COPY:
      grid->editCopy();
      break;
    case ID_EDIT_PASTE:
      grid->editPaste();
      break;
    case ID_EDIT_CLEAR:
      grid->editClear();
      break;
    case ID_VIEW_ZOOM_OUT:
      if(!grid->zoom(Out))
      {
        toolbar->setItemEnabled(ID_VIEW_ZOOM_OUT, false);
        view->setItemEnabled(ID_VIEW_ZOOM_OUT, false);
      }
      break;
    case ID_VIEW_ZOOM_IN:
      grid->zoom(In);
      toolbar->setItemEnabled(ID_VIEW_ZOOM_OUT, true);
      view->setItemEnabled(ID_VIEW_ZOOM_OUT, true);
      break;
    case ID_VIEW_TOGGLE_GRID:
      if(toolbar->isButtonOn(ID_VIEW_TOGGLE_GRID))
        grid->setGrid(true);
      else
        grid->setGrid(false);
      break;
    case ID_DRAW_FREEHAND:
      toggleTool(id);
      grid->setTool(KIconGrid::Freehand);
      break;
    case ID_DRAW_ELLIPSE:
      toggleTool(id);
      grid->setTool(KIconGrid::Ellipse);
      break;
    case ID_DRAW_CIRCLE:
      toggleTool(id);
      grid->setTool(KIconGrid::Circle);
      break;
    case ID_DRAW_RECT:
      toggleTool(id);
      grid->setTool(KIconGrid::Rect);
      break;
    case ID_DRAW_RECT_FILL:
      toggleTool(id);
      grid->setTool(KIconGrid::FilledRect);
      break;
    case ID_DRAW_ERASE:
      toggleTool(id);
      grid->setTool(KIconGrid::Eraser);
      break;
    case ID_DRAW_LINE:
      toggleTool(id);
      grid->setTool(KIconGrid::Line);
      break;
    case ID_DRAW_STRAIGHT:
      toggleTool(id);
      grid->setTool(KIconGrid::Straight);
      break;
    //case ID_DRAW_FIND:
    case ID_DRAW_FILL:
      toggleTool(id);
      grid->setTool(KIconGrid::FloodFill);
      break;
    //case ID_DRAW_SELECT:
    case ID_DRAW_SPRAY:
      toggleTool(id);
      grid->setTool(KIconGrid::Spray);
      break;
    default:
        if(id >= 500 && id <= 600)
          KMsgBox::message(this, i18n("Warning"), i18n("Sorry - not implemented."));
      break;
  }
}

void KIconEdit::updateStatusColors(uint)
{
  QString str;
  str.sprintf(i18n("Colors: %u"), grid->numColors());
  statusbar->changeItem( str.data(), 0);
}

void KIconEdit::updateStatusColors(uint n, uint *)
{
  QString str;
  str.sprintf(i18n("Colors: %u"), n);
  statusbar->changeItem( str.data(), 0);
}

void KIconEdit::updateStatusSize(int x, int y)
{
  QString str;
  str.sprintf("%d x %d", x, y);
  statusbar->changeItem( str.data(), 1);
}

void KIconEdit::updateStatusName(const char *name)
{
  filepath = name;
  QString cap = "";
  if(!filepath.isEmpty())
  {
    cap += filepath;
    cap += " - ";
  }
  cap += kapp->getCaption();
  setCaption(cap.data());
}

void KIconEdit::updateStatusScaling(int s)
{
  QString str;
  str.sprintf("1:%d", s);
  statusbar->changeItem( str.data(), 2);
}

void KIconEdit::toggleTool(int id)
{
  for(int i = ID_DRAW_FREEHAND; i <= ID_DRAW_ERASE; i++)
  {
    if(i != id)
      drawtoolbar->setButton(i, false);
    else
      drawtoolbar->setButton(i, true);
  }
}

void KIconEdit::openBlank(const QSize s)
{
  grid->open( 0, true, s.width(), s.height());
}

void KIconEdit::openTemplate(const QString path)
{
  grid->open( path.data(), true);
}

void KIconEdit::openNewWin(const char *path)
{
  debug("KIconEdit::openNewWin() - %s", path);
  new KIconEdit(0, path);
}


