/*  This file is part of the KDE 
    Copyright (C) 1998 Thomas Tanghus (tanghus@earthling.net)

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/  

#ifndef __KICONEDIT_H__
#define __KICONEDIT_H__

#include <qdir.h>
#include <kapp.h>
#include <ktopwidget.h>
#include <kiconloader.h>
#include <kmenubar.h>
#include <kmsgbox.h>
#include <kstatusbar.h>
#include <ktoolbar.h>
//#include <kfm.h>
//#include <kurl.h>
#include <qwidget.h>
#include <qpixmap.h>
#include <qfileinf.h> 
#include <qlist.h>
#include <qprinter.h>
#include <qkeycode.h>
#include "kicontools.h"
#include "kicongrid.h"
#include "knewicon.h"

#define ID_FILE_NEWWIN  500
#define ID_FILE_NEWFILE 501
#define ID_FILE_OPEN    502
#define ID_FILE_SAVE    503
#define ID_FILE_SAVEAS  504
#define ID_FILE_PRINT   505
#define ID_FILE_CLOSE   506
#define ID_FILE_QUIT    507

#define ID_EDIT_CUT     510
#define ID_EDIT_COPY    511
#define ID_EDIT_PASTE   512
#define ID_EDIT_CLEAR   513
		
#define ID_VIEW_ZOOM_IN     520
#define ID_VIEW_ZOOM_OUT    521
#define ID_VIEW_TOGGLE_GRID 522

#define ID_DRAW_FREEHAND     530
#define ID_DRAW_FILL         531
#define ID_DRAW_RECT         532
#define ID_DRAW_CIRCLE       533
#define ID_DRAW_ELLIPSE      534
#define ID_DRAW_SELECT       535
#define ID_DRAW_SPRAY        536
#define ID_DRAW_RECT_FILL    537
#define ID_DRAW_CIRCLE_FILL  538
#define ID_DRAW_ELLIPSE_FILL 539
#define ID_DRAW_LINE         540
#define ID_DRAW_STRAIGHT     541
#define ID_DRAW_FIND         542
#define ID_DRAW_ERASE        543

class KIconEditView;

/**
* KIconEdit
* @short KClass
* @author Thomas Tanghus <tanghus@earthling.net>
* @version 0.1
*/
class KIconEdit : public KTopLevelWidget
{
    Q_OBJECT
public:
  KIconEdit( const char *name = 0, const char *xpm = 0);
  ~KIconEdit();

  virtual QSize sizeHint();

signals:

public slots:
  virtual void saveProperties(KConfig*);
  virtual void readProperties(KConfig*);
  void unsaved(bool flag);

protected slots:
  void slotActions(int id);
  void updateStatusColors(uint);
  void updateStatusColors(uint, uint*);
  void updateStatusSize(int, int);
  void updateStatusName(const char *);
  void updateStatusScaling(int);
  void openBlank(const QSize);
  void openTemplate(const QString);
  void openNewWin(const char *);

protected:
  KMenuBar *setupMenuBar();
  KToolBar *setupToolBar();
  KToolBar *setupDrawToolBar();
  KStatusBar *setupStatusBar();
  virtual void closeEvent(QCloseEvent*);
  //virtual bool queryExit();
  void toggleTool(int id);

  static QList<KIconEdit> windowlist;
  KIconTemplateContainer *icontemplates;

private:
  //KIconEditView *mainview;
  KMenuBar *menubar;
  KToolBar *toolbar, *drawtoolbar;
  KStatusBar *statusbar;
  KNewIcon *newicon;
  QString filepath;
  QPopupMenu *edit, *file, *view, *help, *tools;
  bool enabletoolbar, enablestatusbar;
  KIconGrid *grid;
  KIconToolsView *toolsw;
  QWidget *mainview;
};



#endif __KICONEDIT_H__



