/*  This file is part of the KDE 
    Copyright (C) 1998 Thomas Tanghus (tanghus@earthling.net)

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/  

#include "kicongrid.moc"

KIconGrid::KIconGrid(QWidget *parent, const char *name, const char *filename)
 : KColorGrid(parent, name, 1)
{
  initMetaObject();
  modified = false;
  img = 0;
  //numcolors = 0;
  currentcolor = qRgb(0,0,0)|OPAQUE_MASK;
  kfm = 0L;
  netfile = "";

  dropZone = new KDNDDropZone( this, DndURL);
  connect( dropZone, SIGNAL( dropAction( KDNDDropZone *) ), 
    this, SLOT( slotDropEvent( KDNDDropZone *) ) );

  filedlg = new KFileDialog( 0, "*.xpm");

  createCursors();
  if(filename)
  {
    filepath = filename;
    if( (filepath.find(":/") != -1) && (filepath.left(1) != "/") )
      open(filename);
    else
    {
      QFileInfo fi(filename);
      filepath = fi.absFilePath();
      open(filepath.data());
    }
  }
  else
  {
    open(0, true);
  }
}

KIconGrid::~KIconGrid()
{
}

void KIconGrid::paintCell( QPainter *painter, int row, int col )
{
  KColorGrid::paintCell(painter, row, col);
  //debug("KIconGrid::paintCell()");

  int d = 0, s = spacing();
  if(hasGrid())
    d = spacing();


  switch( tool )
  {
    case Spray:
    case Eraser:
    case Freehand:
      if ( row * numCols() + col == selected)
        painter->drawWinFocusRect( d, d, cellWidth()-(2*d), cellHeight()-(2*d) );
      break;
    case FilledRect:
    case Rect:
    case Ellipse:
    case Circle:
    case Line:
    case Straight:
      if(btndown && isMarked(col, row))
        painter->drawWinFocusRect( d+s, d+s, cellWidth()-(2*(d+s)), cellHeight()-(2*(d+s)) );
      break;
    default:
      break;
  }

  //debug("KIconGrid::paintCell() - done");
}

void KIconGrid::setColor( int cell, uint color)
{
  KColorGrid::setColor(cell, color);
  modified = true;    
}

void KIconGrid::mousePressEvent( QMouseEvent *e )
{
  int row = findRow( e->pos().y() );
  int col = findCol( e->pos().x() );
  //int cell = row * numCols() + col;

  if(!img->valid(col, row))
    return;

  btndown = true;
  start.setX(col);
  start.setY(row);
}

void KIconGrid::mouseMoveEvent( QMouseEvent *e )
{
  int row = findRow( e->pos().y() );
  int col = findCol( e->pos().x() );
  int cell = row * numCols() + col;

  if(!img->valid(col, row) || !btndown)
    return;
  end.setX(col);
  end.setY(row);

  switch( tool )
  {
    case Eraser:
      currentcolor = TRANSPARENT;
    case Freehand:
    {
      setColor( cell, currentcolor );
  //img->setPixel(col, row, currentcolor.pixel());

      if ( selected != cell )
      {
        modified = true;
        int prevSel = selected;
        selected = cell;
        updateCell( prevSel/numCols(), prevSel%numCols(), FALSE );
        updateCell( row, col, FALSE );
        *((uint*)img->scanLine(row) + col) = (colorAt(cell));
      }
      break;
    }
    case Ellipse:
    case Circle:
    {
      drawEllipse(false);
      break;
    }
    case FilledRect:
    case Rect:
    {
      drawRect(false);
      break;
    }
    case Straight:
    {
      drawStraightLine(false);
      break;
    }
    case Line:
    {
      drawLine(false);
      break;
    }
    case Spray:
    {
      drawSpray(QPoint(col, row));
      break;
    }
    default:
      break;
  }

  p = *img;

  emit changed(QPixmap(p));
  //emit colorschanged(numColors(), data());
}

void KIconGrid::mouseReleaseEvent( QMouseEvent *e )
{
  int row = findRow( e->pos().y() );
  int col = findCol( e->pos().x() );
  btndown = false;
  end.setX(col);
  end.setY(row);
  int cell = row * numCols() + col;

  switch( tool )
  {
    case Eraser:
      currentcolor = TRANSPARENT;
    case Freehand:
    {
      if(!img->valid(col, row))
        return;
      setColor( cell, currentcolor );
      if ( selected != cell )
      {
        modified = true;
        int prevSel = selected;
        selected = cell;
        updateCell( prevSel/numCols(), prevSel%numCols(), FALSE );
        updateCell( row, col, FALSE );
        *((uint*)img->scanLine(row) + col) = colorAt(cell);
        p = *img;
      }
      break;
    }
    case Ellipse:
    case Circle:
    {
      drawEllipse(true);
      break;
    }
    case FilledRect:
    case Rect:
    {
      drawRect(true);
      break;
    }
    case Straight:
    {
      drawStraightLine(true);
      break;
    }
    case Line:
    {
      drawLine(true);
      break;
    }
    case Spray:
    {
      drawSpray(QPoint(col, row));
      break;
    }
    default:
      break;
  }
  emit changed(QPixmap(p));
  //emit colorschanged(numColors(), data());
}


void KIconGrid::resizeEvent(QResizeEvent *ev)
{
  QTableView::resizeEvent(ev);
}

QSize KIconGrid::sizeHint()
{
  //return QSize(totalWidth(), totalHeight()); // doesn't give expected result
  return QSize(cellWidth()*numCols(), cellWidth()*numRows()); // neither does this :-(
}

//void KIconGrid::setColorSelection( const QColor &color )
void KIconGrid::setColorSelection( uint c )
{
  currentcolor = c;
}

void KIconGrid::save( const char *filename )
{
  QString tmpname;
  if(strlen(filename) == 0)
  {
    tmpname = filedlg->getSaveFileName(lastdir.data(), "*.xpm");
    if(tmpname.length() == 0)
      return;
    QFileInfo fi(tmpname.data());
    lastdir = fi.dirPath(true).copy();
    filepath = tmpname.copy();
    filename = tmpname.data();
    debug("KIcongrid::save(%s)", filename);
    emit namechanged(filepath.data());
  }
  debug("img colors: %d", numColors());
  QImage *tmp;
  img->setAlphaBuffer(true);
  tmp = new QImage(img->convertDepth(8, ColorOnly|AvoidDither));
  debug("tmp colors: %d", tmp->numColors());
  //for(int i = 0; i < tmp->numColors(); i++)
  //  debug("Color: %d %d %d",
  //           qRed(tmp->color(i)), qGreen(tmp->color(i)), qBlue(tmp->color(i)));
  tmp->setAlphaBuffer(true);

  QImageIO iio;
  iio.setImage(*tmp);
  iio.setFileName(filename);
  iio.setFormat("XPM");
  iio.setDescription("Created by kiconedit");
  
  if(tmp->save(filename, "XPM")) //iio.write())
  {
    filepath = filename;
    modified = false;
  }
  else
  {
    QString msg = i18n("There was an error saving:\n");
    msg += filename;
    msg += "\n";
    KMsgBox::message(this, i18n("Warning"), msg.data());
  }
  delete tmp;
}

void KIconGrid::open( const char *filename, bool newfile, int w, int h )
{
  QString target;
  QString file = filename;

  debug("Opening %s", file.data());
  bool istemplate = false, isblank = false;
  if(newfile && file.isEmpty())
    isblank = true;
  if(newfile && !file.isEmpty())
    istemplate = true;

  if(!newfile && file.isEmpty())
  {
    //filedlg->setDir((const char*)lastdir.data(), true);
    file = filedlg->getOpenFileName(lastdir.data(), "*.xpm").copy();
    lastdir = filedlg->dirPath().copy();
    if(file.length() == 0)
      return;
    QFileInfo fi(file.data());
    lastdir = fi.dirPath(true).copy();
    filename = file.data();
    //debug(file.data());
  }

  if(img)
    delete img;

  if(!isblank)
  {
    if(kfm != 0L)
    {
      KMsgBox::message (this, i18n("Error"), 
	    i18n("The Icon Editor is already loading a file.\nWait until this one is finished."));
      return;
    }
    kfm = new KFM;  
    CHECK_PTR(kfm);
    if(kfm->download(file, target))
    {
      file = target;
    }
    else
    {
      kfm->removeTempFile(target);
      delete kfm;
      kfm = 0L;  
      QString msg = i18n("There was an error loading:\n") + file;
      KMsgBox::message (this, i18n("Error"), msg.data());
      return;
    }
    img = new QImage;
    if(!img->load(file.data(), "XPM"))
    {
      delete kfm;
      kfm = 0L;  
      QString msg = i18n("There was an error loading:\n");
      msg += file;
      msg += "\n";
      KMsgBox::message(this, i18n("Warning"), msg.data());
      open(0, true, 32, 32);
      return;
    }
    kfm->removeTempFile(target);
    delete kfm;
    kfm = 0L;  

    load(img);
  }
  else
  {
    img = new QImage(w, h, 32);
    img->fill(TRANSPARENT);
    setNumRows(h);
    setNumCols(w);
    fill(TRANSPARENT);
  }

  if(isblank || istemplate)
    filepath = "";
  else
    filepath = file.copy();
  modified = false;

  p = *img;
  emit changed(QPixmap(p));
  emit sizechanged(numCols(), numRows());
  //emit colorschanged(numColors(), data());
  emit namechanged(filepath.data());
  repaint();
}

void KIconGrid::load( const QImage *image )
{
  const QImage *tmp = image;
  img = new QImage(image->convertDepth(32));
  img->setAlphaBuffer(true);
  delete tmp;

  setNumRows(img->height());
  setNumCols(img->width());

  for(int y = 0; y < numRows(); y++)
  {
    uint *l = (uint*)img->scanLine(y);
    for(int x = 0; x < numCols(); x++, l++)
    {
      uint gray = (qRgb(200, 200, 200) | OPAQUE_MASK);
      if(*l == gray)  // this is a hack but I couldn't save it as transparent otherwise
      {
        *l = TRANSPARENT;
      }
      setColor((y*numCols())+x, *l);
    }
  }
}

void KIconGrid::slotDropEvent( KDNDDropZone * _dropZone )
{
  debug("dropEvent");
  QStrList & list = _dropZone->getURLList();
  char *s;
  
  for ( s = list.first(); s != 0L; s = list.next() )
  {
    // Load the first file in this window
    debug("url: %s", s);
    if ( s == list.getFirst() && !isModified() ) 
    {
      QString n = s;
      if ( n.left(5) == "file:" || n.left(4) == "ftp:" )
        open( n.data() );
    }
    else 
    {
      emit opennewwin(s);
      //KIconEdit *e = new KIconEdit(s.data());
    }
  }    
}

const QPixmap &KIconGrid::pixmap()
{
  if(!img->isNull())
    p = *img;
  //p.convertFromImage(*img, 0);
  return(p);
}

bool KIconGrid::zoom(Direction d)
{
  int f = (d == In) ? (cellWidth()+1) : (cellWidth()-1);
  setCellWidth( f );
  setCellHeight( f );
  repaint();
  emit scalingchanged(cellWidth());
  if(d == Out && cellWidth() <= 1)
    return false;
  return true;
}

void KIconGrid::editClear()
{
  img->fill(TRANSPARENT);
  fill(TRANSPARENT);
  update();
  modified = true;
  p = *img;
  emit changed(p);
}

void KIconGrid::editCut()
{
  editCopy();
  editClear();
}

void KIconGrid::editCopy()
{
  img->setAlphaBuffer(true);
  QImage tmp(img->convertDepth(8));
  tmp.setAlphaBuffer(true);
  QString str = "";
  QBuffer buf(str);
  buf.open(IO_WriteOnly);
  QImageIO iio;
  iio.setImage(tmp);
  iio.setIODevice(&buf);
  iio.setFormat("XPM");
  iio.setDescription("Created by kiconedit");
  
  if(iio.write())
  {
    QClipboard *cb = kapp->clipboard();
    cb->setText(str.data());
    //debug(str.data());
  }
  else
  {
    QString msg = i18n("There was an copying:\n");
    msg += filepath;
    msg += "\n to the clipboard!";
    KMsgBox::message(this, i18n("Warning"), msg.data());
  }
  buf.close();
}

void KIconGrid::editPaste()
{
  QClipboard *cb = kapp->clipboard();
  QString str = cb->text();
  QBuffer buf(str);
  buf.open(IO_ReadOnly);
  QImageIO iio;
  iio.setIODevice(&buf);
  iio.setFormat("XPM");
  
  if(iio.read())
  {
    if(img)
      delete img;
    img = new QImage(iio.image());
    load(img);
  }
  else
  {
    QString msg = i18n("Invalid pixmap data in clipboard!\n");
    KMsgBox::message(this, i18n("Warning"), msg.data());
  }
  buf.close();
  filepath = "";
  modified = false;

  p = *img;
  emit changed(QPixmap(p));
  emit sizechanged(numCols(), numRows());
  emit colorschanged(numColors(), data());
  emit namechanged(filepath.data());

}

void KIconGrid::createCursors()
{
  KIconLoader *loader = kapp->getIconLoader();
  QBitmap bmp, mask(22, 22);
  QPixmap pix;

  cursor_normal = QCursor(arrowCursor);

  pix = loader->loadIcon("flood.xpm");
  mask = pix.createHeuristicMask();
  bmp = pix;
  cursor_flood = QCursor(bmp, mask);

  pix = loader->loadIcon("aim.xpm");
  if(pix.isNull())
    debug("Error loading pixmap: aim.xpm"); 
  mask = pix.createHeuristicMask();
  bmp = pix;
  cursor_aim = QCursor(bmp, mask);

  pix = loader->loadIcon("spraycan.xpm");
  mask = pix.createHeuristicMask(true);
  bmp = pix;
  cursor_spray = QCursor(bmp, mask);

  pix = loader->loadIcon("eraser.xpm");
  mask = pix.createHeuristicMask(true);
  bmp = pix;
  cursor_erase = QCursor(bmp, mask);

  //setCursor(cursor_spray);
}

void KIconGrid::setTool(DrawTool t)
{
  btndown = false;
  tool = t;

  switch( tool )
  {
    case Line:
      //KMsgBox::message(this, i18n("Warning"), i18n("Sorry - not implemented."));
    case Freehand:
    case Straight:
    case Ellipse:
    case Circle:
    case FilledRect:
    case Rect:
      setCursor(cursor_aim);
      break;
    case Spray:
      setCursor(cursor_spray);
      break;
    case Eraser:
      setCursor(cursor_erase);
      currentcolor = TRANSPARENT;
      break;
    case FloodFill:
      setCursor(cursor_flood);
      KMsgBox::message(this, i18n("Warning"), i18n("Sorry - not implemented."));
      break;
    default:
      break;
  }
  ((QScrollBar*)verticalScrollBar())->setCursor(arrowCursor);
  ((QScrollBar*)horizontalScrollBar())->setCursor(arrowCursor);
}

void KIconGrid::drawSpray(QPoint point)
{
  int x = (point.x()-5);
  int y = (point.y()-5);
  //debug("drawSpray() - %d X %d", x, y);
  
  pntarray.resize(0);
  int points = 0;
  for(int i = 1; i < 4; i++, points++)
  {
    int dx = (rand() % 10);
    int dy = (rand() % 10);
    pntarray.putPoints(points, 1, x+dx, y+dy);
  }

  drawPointArray(pntarray, Draw);
}

void KIconGrid::drawEllipse(bool drawit)
{
  if(drawit)
  {
    drawPointArray(pntarray, Draw);
    p = *img;
    emit changed(p);
    repaint();
    return;
  }

  QPointArray a(pntarray.copy());
  int x = start.x(), y = start.y(), cx, cy;
  //cx = (x > end.x()) ? (x - end.x()) : (end.x() - x);
  if(x > end.x())
  {
    cx = x - end.x();
    x = x - cx;
  }
  else
    cx = end.x() - x;
  if(y > end.y())
  {
    cy = y - end.y();
    y = y - cy;
  }
  else
    cy = end.y() - y;
  //cy = (y > end.y()) ? (y - end.y()) : (end.y() - y);
  int d = (cx > cy) ? cx : cy;

  //debug("%d, %d - %d %d", x, y, d, d);
  pntarray.resize(0);
  drawPointArray(a, Mark);
  //repaint();
  if(tool == Circle)
    pntarray.makeEllipse(x, y, d, d);
  else if(tool == Ellipse)
    pntarray.makeEllipse(x, y, cx, cy);
  drawPointArray(pntarray, Mark);
}

void KIconGrid::drawRect(bool drawit)
{
  if(drawit)
  {
    drawPointArray(pntarray, Draw);
    p = *img;
    emit changed(p);
    repaint();
    return;
  }

  QPointArray a(pntarray.copy());
  int x = start.x(), y = start.y(), cx, cy;
  //cx = (x > end.x()) ? (x - end.x()) : (end.x() - x);
  if(x > end.x())
  {
    cx = x - end.x();
    x = x - cx;
  }
  else
    cx = end.x() - x;
  if(y > end.y())
  {
    cy = y - end.y();
    y = y - cy;
  }
  else
    cy = end.y() - y;

  //debug("%d, %d - %d %d", x, y, cx, cy);
  pntarray.resize(0);
  drawPointArray(a, Mark); // remove previous marking

  int points = 0;

  if(tool == FilledRect)
  {
    for(int i = x; i < x+cx; i++)
    {
      for(int j = y; j < y+cy; j++, points++)
        pntarray.putPoints(points, 1, i, j);
    }
  }
  else
  {
    for(int i = x; i <= x+cx; i++, points++)
      pntarray.putPoints(points, 1, i, y);
    for(int i = y; i <= y+cy; i++, points++)
      pntarray.putPoints(points, 1, x, i);
    for(int i = x; i <= x+cx; i++, points++)
      pntarray.putPoints(points, 1, i, y+cy);
    for(int i = y; i <= y+cy; i++, points++)
      pntarray.putPoints(points, 1, x+cx, i);
  }

  drawPointArray(pntarray, Mark);
}

void KIconGrid::drawStraightLine(bool drawit)
{
  if(drawit)
  {
    drawPointArray(pntarray, Draw);
    p = *img;
    emit changed(p);
    repaint();
    return;
  }

  QPointArray a(pntarray.copy());
  int x = start.x(), y = start.y(), cx = end.x(), cy = end.y();
  int ydiff, xdiff;
  bool yasc = true;
  bool xasc = true;
  if(x > cx)
  {
    xasc = false;
    xdiff = x - cx;
  }
  else
    xdiff = cx - x;
  if(y > cy)
  {
    yasc = false;
    ydiff = y - cy;
  }
  else
    ydiff = cy - y;

  //debug("%d, %d - %d %d", x, y, cx, cy);
  pntarray.resize(0);
  drawPointArray(a, Mark); // remove previous marking

  int points = 0;

  if(xdiff > ydiff)
  {
    for(int i = (xasc ? x : cx); i < (xasc ? cx : x); i++, points++)
      pntarray.putPoints(points, 1, i, y);
  }
  else
  {
    for(int i = (yasc ? y : cy); i < (yasc ? cy : y); i++, points++)
      pntarray.putPoints(points, 1, x, i);
  }

  
  drawPointArray(pntarray, Mark);
}

static
bool pnt_on_line( QPoint p, QPoint q, QPoint t )
{
  return true;
}


void KIconGrid::drawLine(bool drawit)
{
  if(drawit)
  {
    drawPointArray(pntarray, Draw);
    p = *img;
    emit changed(p);
    repaint();
    return;
  }

  QPointArray a(pntarray.copy());
  int x = start.x(), y = start.y(), cx = end.x(), cy = end.y();
  int ydiff, xdiff;
  bool yasc = true;
  bool xasc = true;
  if(x > cx)
  {
    xasc = false;
    xdiff = x - cx;
  }
  else
    xdiff = cx - x;
  if(y > cy)
  {
    yasc = false;
    ydiff = y - cy;
  }
  else
    ydiff = cy - y;

  //debug("%d, %d - %d %d", x, y, cx, cy);
  pntarray.resize(0);
  drawPointArray(a, Mark); // remove previous marking

  int points = 0;

  for(int i = (xasc ? x : cx); i < (xasc ? cx : x); i++)
  {
    for(int j = (yasc ? y : cy); j < (yasc ? cy : y); j++, points++)
    {
      if(pnt_on_line(start, end, QPoint(i,j)))
        pntarray.putPoints(points, 1, i, j);
    }
  }  
  drawPointArray(pntarray, Mark);
}

void KIconGrid::drawPointArray(QPointArray a, DrawAction action)
{
  QRect rect = a.boundingRect();

  int s = a.size(); //((rect.size().width()) * (rect.size().height()));
  for(int i = 0; i < s; i++)
  {
    int x = a[i].x();
    int y = a[i].y();
    //if(img->valid(x, y) && !QSize(x, y).isNull() && rect.contains(QPoint(x, y)))
    if(img->valid(x, y) && rect.contains(QPoint(x, y)))
    {
      //debug("x: %d - y: %d", x, y);
      switch( action )
      {
        case Draw:
        {
          *((uint*)img->scanLine(y) + x) = currentcolor; //colors[cell]|OPAQUE;
          int cell = y * numCols() + x;
          setColor( cell, currentcolor );
          //updateCell( y, x, FALSE );
          break;
        }
        case Mark:
        case UnMark:
          updateCell( y, x, true );
          break;
        default:
          break;
      }
    }
  }

}

bool KIconGrid::isMarked(QPoint point)
{
  return isMarked(point.x(), point.y());
}

bool KIconGrid::isMarked(int x, int y)
{
  //QRect rect = pntarray.boundingRect();

  if(((y * numCols()) + x) == selected)
    return true;

  int s = pntarray.size(); //((rect.size().width()) * (rect.size().height()));
  for(int i = 0; i < s; i++)
  {
    int px = pntarray[i].x();
    int py = pntarray[i].y();
    if(y == py && x == px)
      return true;
  }

  return false;
}


