/*  This file is part of the KDE 
    Copyright (C) 1998 Thomas Tanghus (tanghus@earthling.net)

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/  

#ifndef __KICONGRID_H__
#define __KICONGRID_H__

#include <qtablevw.h>
#include <qdrawutl.h>
#include <qbrush.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qbitmap.h>
#include <qbuffer.h>
#include <qclipbrd.h>
#include <qscrbar.h>
#include <qfileinf.h>
#include <kfiledialog.h> 
#include <qcursor.h>
#include <qpntarry.h>
#include <qstrlist.h>
#include <kcolordlg.h>
#include <kiconloader.h>
#include <kapp.h>
#include <kfm.h>
#include <kurl.h>
#include <kmsgbox.h>
#include "colors.h"
#include "kcolorgrid.h"

enum Direction { In, Out };

/**
* KIconGrid
* @short KIconGrid
* @author Thomas Tanghus <tanghus@earthling.net>
* @version 0.1
*/
class KIconGrid : public KColorGrid
{
    Q_OBJECT
public:
  KIconGrid( QWidget * parent = 0, const char * name = 0, const char * filename = 0);
  virtual ~KIconGrid();

  enum DrawTool { Straight, Line, Freehand, FloodFill, Spray, Rect, FilledRect, Circle, Ellipse, Eraser };
  enum DrawAction { Mark, UnMark, Draw };
  enum Action { GET, PUT };

  bool isModified() { return modified; };
  void save(const char *filename);
  void open( const char *file = 0, bool newfile = false, int w = 32, int h = 32 );
  void load( const QImage * );
  //void openURL(const char *file);
  void editCut();
  void editCopy();
  void editPaste();
  void setTool(DrawTool tool);
  const QPixmap &pixmap();
  bool zoom(Direction direct);
  void editClear();
  virtual QSize sizeHint();
  int rows() { return numRows(); };
  int cols() { return numCols(); };
  uint getColors( uint *_colors) { return colors(_colors); }
  bool isMarked(QPoint p);
  bool isMarked(int x, int y);
  const char *title() { return filepath.data(); }
  int scaling() { return cellWidth(); }

signals:
  void scalingchanged(int);
  void changed( const QPixmap & );
  void sizechanged( int, int );
  void colorschanged( uint, uint* );
  void namechanged( const char * );
  void addingcolor(uint);
  void opennewwin(const char *);

protected slots:
  void setColorSelection( uint );
  void slotDropEvent( KDNDDropZone * _dropZone );

protected:
  virtual void paintCell( QPainter*, int, int );
  virtual void resizeEvent(QResizeEvent*);
  virtual void mousePressEvent(QMouseEvent*);
  virtual void mouseReleaseEvent(QMouseEvent*);
  virtual void mouseMoveEvent(QMouseEvent*);
  void setColor(int colNum, uint );
  void createCursors();
  void drawPointArray(QPointArray, DrawAction);
  void drawEllipse(bool);
  void drawLine(bool);
  void drawStraightLine(bool);
  void drawRect(bool);
  void drawSpray(QPoint);

  uint currentcolor;
  QPoint start, end;
  QImage *img;
  QPixmap p;
  QString filepath, url, lastdir, netfile, tmpfile;
  int selected, tool; //, numrows, numcols;
  bool modified, btndown;
  QPointArray pntarray;
  QCursor cursor_normal, cursor_aim, cursor_flood, cursor_spray, cursor_erase;
  KFM *kfm;
  Action kfmaction;
  KDNDDropZone *dropZone;
  KFileDialog *filedlg;
};



#endif __KICONGRID_H__



